package cn.com.mjsoft.cms.content.service;

import ch.qos.logback.classic.Logger;
import cn.com.mjsoft.cms.cluster.adapter.ClusterMapAdapter;
import cn.com.mjsoft.cms.common.spring.SpringContextUtils;
import cn.com.mjsoft.cms.organization.bean.SystemOrganizationBean;
import cn.com.mjsoft.cms.organization.service.OrgService;
import cn.com.mjsoft.cms.security.bean.SystemUserBean;
import cn.com.mjsoft.cms.security.service.SecurityService;
import cn.com.mjsoft.framework.security.Auth;
import cn.com.mjsoft.framework.security.session.SecuritySession;
import cn.com.mjsoft.framework.security.session.SecuritySessionKeeper;
import cn.com.mjsoft.framework.util.DateAndTimeUtil;
import cn.com.mjsoft.framework.util.StringUtil;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 内容唯一维护权服务
 */
@Service
public class ExtContentMTService
{
    private static final Logger log = (Logger) LoggerFactory.getLogger( ExtContentMTService.class );

    public static ExtContentMTService getInstance()
    {
        return SpringContextUtils.getBean("extContentMTService");
    }

    //内容维护权记录
    private static ClusterMapAdapter CONTENT_MT = new ClusterMapAdapter(
            "contentService.CONTENT_MT", Long.class, String.class );


    /**
     * 设置当前登录管理员对内容ID的维护权,保持唯一一个管理员在编辑。
     * 需由心跳模式更新时间，后台线程监视心跳，若大1分钟不更新，视为
     * 自动退出
     * @param userName
     * @param contentId
     */
    public void setContentMT(String contentId)
    {
        //获取当前登录管理员
        Auth auth = SecuritySessionKeeper.getSecuritySession().getAuth();

        if(auth == null)
        {
            return;
        }

        Long userId = (Long) auth.getIdentity();

        String mtUserId = (String) CONTENT_MT.get(contentId);

        if( mtUserId == null || mtUserId.startsWith(userId+":"))
        {
            CONTENT_MT.put(contentId, userId+":"+ DateAndTimeUtil.clusterTimeMillis());
        }
    }

    /**
     * 返回指定内容ID可能存在的维护userName
     * @param contentId
     * @return
     */
    public String getContentInMT( String contentId )
    {
        //获取当前登录管理员
        Auth auth = SecuritySessionKeeper.getSecuritySession().getAuth();

        if(auth == null)
        {
            return null;
        }

        String val = (String) CONTENT_MT.get(contentId);

        if(val != null)
        {
            return val.split(":")[0];
        }

        return null;
    }

    /**
     * 监视状态，一分钟不更新的维护权内容ID将释放，此方法由定时线程调用（分钟）
     */
    public void checkAndClearContentMT()
    {
        Long currDT = DateAndTimeUtil.clusterTimeMillis();

        //需要临时Map，删除用
        Map tmp = new HashMap();

        tmp.putAll(CONTENT_MT.getMap());

        Iterator iter =  tmp.entrySet().iterator();

        Map.Entry<String, String> ent = null;

        Long dt = null;

        String val = null;

        while( iter.hasNext() )
        {
            ent = (Map.Entry<String, String>) iter.next();

            val = ent.getValue();

            dt = Long.valueOf(val.split(":")[1]);

            long seconds = TimeUnit.MILLISECONDS.toSeconds(currDT - dt);

            //超过60秒删除
            if(seconds >= 60)
            {
                CONTENT_MT.remove(ent.getKey());
            }
        }

        return;
    }

    /**
     * 清空当前contentId维护锁信息
     * @param contentId
     */
    public void clearContentMT(String contentId)
    {
        //获取当前登录管理员
        Auth auth = SecuritySessionKeeper.getSecuritySession().getAuth();

        if(auth == null)
        {
            return;
        }

        Long userId = (Long) auth.getIdentity();

        String mtUserId = (String) CONTENT_MT.get(contentId);

        if( mtUserId == null || mtUserId.startsWith(userId+":"))
        {
            CONTENT_MT.remove(contentId);
        }
    }

    /**
     * 返回管理员信息
     * @param userId
     * @return Map<String, String>
     */
    public Map<String, String> getManagerUserInfo( Long userId )
    {
        //获取当前管理员权限识别
        SecuritySession sec = SecuritySessionKeeper.getSecuritySession();

        if(sec == null || !sec.isManager())
        {
            return null;
        }

        Auth auth = sec.getAuth();

        if(auth == null)
        {
            return null;
        }

        Map<String, String> res = new HashMap<>();

        SystemUserBean user = SecurityService.getInstance()
                .retrieveSingleSystemUserBean(null, userId);

        if(user != null)
        {
            res.put("userName", user.getUserName());
            res.put("userTrueName", user.getUserTrueName());

            SystemOrganizationBean org =OrgService.getInstance()
                    .retrieveSingleOrganizationBeanByOrgIdLinearFlag(user.getRelateOrgCode());

            if(org != null)
            {
                res.put("orgName", org.getOrgName());
            }
        }

        return res;
    }

}
